/**************************************************************************
*  Copyright (c) 2013-2018 by Michael Fischer (www.emb4fun.de).
*  All rights reserved.
*
*  Redistribution and use in source and binary forms, with or without 
*  modification, are permitted provided that the following conditions 
*  are met:
*  
*  1. Redistributions of source code must retain the above copyright 
*     notice, this list of conditions and the following disclaimer.
*  2. Redistributions in binary form must reproduce the above copyright
*     notice, this list of conditions and the following disclaimer in the 
*     documentation and/or other materials provided with the distribution.
*  3. Neither the name of the author nor the names of its contributors may 
*     be used to endorse or promote products derived from this software 
*     without specific prior written permission.
*
*  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
*  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
*  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS 
*  FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL 
*  THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, 
*  INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, 
*  BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS 
*  OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED 
*  AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, 
*  OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF 
*  THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF 
*  SUCH DAMAGE.
*
***************************************************************************
*  History:
*
*  19.05.2013  mifi  First Version.
*  19.08.2013  mifi  Some rework and cleanup.
*  18.04.2014  mifi  Some rework, HW init of ETH will be done by
*                    ethernetif_init.
*  04.03.2018  mifi  Added function to retrieve the MAC address of
*                    the board.
**************************************************************************/
#define __IPSTACK_C__

/*=======================================================================*/
/*  Include                                                              */
/*=======================================================================*/
#include <string.h>
#include "project.h"
#include "ethernetif.h"

#include "lwip\api.h"
#include "lwip\tcpip.h"
#include "lwip\inet.h"
#include "lwip\netif.h"

/*=======================================================================*/
/*  Extern                                                               */
/*=======================================================================*/

/*=======================================================================*/
/*  Global                                                               */
/*=======================================================================*/

int errno = 0; /* Needed by lwIP */

/*=======================================================================*/
/*  All Structures and Common Constants                                  */
/*=======================================================================*/

/*=======================================================================*/
/*  Definition of all local Data                                         */
/*=======================================================================*/

static struct netif NetIf;

/*=======================================================================*/
/*  Definition of all local Procedures                                   */
/*=======================================================================*/

/*************************************************************************/
/*  BoardGetMACAddress                                                   */
/*                                                                       */
/*  Retrieve the MAC address of the board.                               */
/*  In case of an error, a default address will be used.                 */
/*                                                                       */
/*  In    : pAddress                                                     */
/*  Out   : pAddress                                                     */
/*  Return: none                                                         */
/*************************************************************************/
static void BoardGetMACAddress (uint8_t *pAddress)
{
   TAL_RESULT      Error         = TAL_ERROR;
   static uint8_t bMACRetrieved  = TAL_FALSE;
   static uint8_t  MACAddress[6] = {0x00,0x11,0x22,0x33,0x44,0x55};
   uint8_t         Buffer[6];
   int             ReadCnt;
   
   if (TAL_FALSE == bMACRetrieved)
   {
      bMACRetrieved = TAL_TRUE;

      /* Sets up board specific I2C interface */
      Chip_IOCON_PinMuxSet(LPC_IOCON, 0, 27, IOCON_FUNC1);
      Chip_IOCON_PinMuxSet(LPC_IOCON, 0, 28, IOCON_FUNC1);
   
      /* Initialize I2C */
      Chip_I2C_Init(I2C0);
      Chip_I2C_SetClockRate(I2C0, 100000);

      /* Set I2C mode to polling */
      Chip_I2C_SetMasterEventHandler(I2C0, Chip_I2C_EventHandlerPolling);
      
      /* Get MAC address from the onboard EEprom (0xA0>>1 = 0x50) at offset 0xFA */
      ReadCnt = Chip_I2C_MasterCmdRead(I2C0, 0x50, 0xFA, Buffer, 6);
      if (6 == ReadCnt)
      {
         memcpy(MACAddress, Buffer, 6);
         Error = TAL_OK;
      }
      
      Chip_I2C_DeInit(I2C0);
   }
   
   /* Return MAC address */
   memcpy(pAddress, MACAddress, 6);
   
} /* BoardGetMACAddress */

/*=======================================================================*/
/*  All code exported                                                    */
/*=======================================================================*/

/*************************************************************************/
/*  IPInit                                                               */
/*                                                                       */
/*  In    : none                                                         */
/*  Out   : none                                                         */
/*  Return: none                                                         */
/*************************************************************************/
void IPInit (void)
{
   uint8_t   MACAddress[6];
   ip_addr_t ipaddr;
   ip_addr_t netmask;
   ip_addr_t gw;  

   /* Setup MAC address */
   BoardGetMACAddress(MACAddress);
   
   /* IP address setting */
   ipaddr.addr  = inet_addr(IP_DEFAULT_ADDRESS);
   netmask.addr = inet_addr(IP_DEFAULT_NETMASK);
   gw.addr      = inet_addr(IP_DEFAULT_GATEWAY); 
   
   /* Initialize lwIP */
   tcpip_init(NULL, NULL);

   /* Add the network interface to the list of lwIP netifs. */
   netif_add(&NetIf, &ipaddr, &netmask, &gw, MACAddress, ethernetif_init, tcpip_input);
   
   /* Set the network interface as the default network interface. */
   netif_set_default(&NetIf);

   /* Bring the interface up. */
   netif_set_up(&NetIf);
   
} /* IPInit */

/*************************************************************************/
/*  IPIsNetIfUp                                                          */
/*                                                                       */
/*  Check if the interface and link is Up.                               */
/*                                                                       */
/*  In    : none                                                         */
/*  Out   : none                                                         */
/*  Return: 0 / 1                                                        */
/*************************************************************************/
int IPIsNetIfUp (void)
{
   int NetIfUp = 0;
   
   if ((netif_is_up(&NetIf)) && (netif_is_link_up(&NetIf)))
   {
      NetIfUp = 1;
   }
   
   return(NetIfUp);
} /* IPIsNetIfUp */

/*** EOF ***/
